@@ -118,6 +118,7 @@ doCallback (ClientData clientData, Tcl_Interp * interp, int argc, CONST84 char *
 {
   OOP oop, *args;
   int i;
+  char result[2] = "0";
 
   /* argv[0] is "callback", the command name */
   args = alloca (sizeof (OOP) * (argc - 2));
@@ -128,10 +129,9 @@ doCallback (ClientData clientData, Tcl_Interp * interp, int argc, CONST84 char *
   args[argc - 3] = NULL;
   if (vmProxy->vmsgSend (oop, vmProxy->symbolToOOP (argv[2]), args)
       == vmProxy->trueOOP)
-    Tcl_SetResult (interp, "1", TCL_STATIC);
-  else
-    Tcl_SetResult (interp, "0", TCL_STATIC);
+    result[0]++;
 
+  Tcl_SetResult (interp, result, TCL_VOLATILE);
   return TCL_OK;
 }
 
@@ -179,8 +179,10 @@ gst_initModule (VMProxy * proxy)
 
 typedef struct
 {
-  char *cptr;
-  char *Bcmt, *Ecmt, Bos, Eos;
+  const char *cptr;
+  const char *Bcmt;
+  const char *Ecmt;
+  char Bos, Eos;
   mst_Boolean xpm1;		/* 1 if XPM1, 0 otherwise */
 }
 XpmData;
@@ -190,7 +192,7 @@ XpmData;
 /* number of xpmColorKeys */
 #define NKEYS 5
 
-char *xpmColorKeys[] = {
+const char *xpmColorKeys[] = {
   "s",				/* key #1: symbol */
   "m",				/* key #2: mono visual */
   "g4",				/* key #3: 4 grays visual */
@@ -200,9 +202,9 @@ char *xpmColorKeys[] = {
 
 typedef struct
 {
-  char *type;			/* key word */
-  char *Bcmt;			/* string beginning comments */
-  char *Ecmt;			/* string ending comments */
+  const char *type;			/* key word */
+  const char *Bcmt;			/* string beginning comments */
+  const char *Ecmt;			/* string ending comments */
   char Bos;			/* character beginning strings */
   char Eos;			/* character ending strings */
 }
@@ -329,7 +331,7 @@ xpmParseComment (XpmData * mdata)
 {
   register char c;
   register unsigned int n = 0;
-  char *s2;
+  const char *s2;
 
   /* skip the string beginning comment */
   s2 = mdata->Bcmt;
@@ -577,7 +579,7 @@ xpmParseValues (XpmData * data, unsigned int *width, unsigned int *height,
 	    {
 	      hotspot = (atoui (buf, l, &x_hotspot)
 			 && xpmNextUI (data, &y_hotspot));
-	      l = xpmNextWord (data, buf, BUFSIZ);
+ 	      l = xpmNextWord (data, buf, BUFSIZ);
 	    }
 	}
     }
@@ -639,7 +641,8 @@ xpmParseColors (XpmData * data, unsigned int ncolors, unsigned int cpp,
   unsigned int lastwaskey;	/* key read */
   char buf[BUFSIZ + 1];
   char curbuf[BUFSIZ + 1];	/* current buffer */
-  char **sptr, *s;
+  const char **sptr;
+  char *s;
   XpmColor *color;
   XpmColor *colorTable;
   char **defaults;
@@ -1147,7 +1150,7 @@ void
 xpmInit (void)
 {
   static Tk_PhotoImageFormat xpmFormat = {
-    "XPM",
+    (char *) "XPM",
     xpmFileMatch,
     xpmStringMatch,
     xpmFileRead,
