@@ -486,7 +486,7 @@ invoke_smalltalk_closure(GClosure     *closure,
 	}
       args[i] = oop;
     }
-  args[n_param_values + 1] = stc->data;
+  args[n_param_values] = stc->data;
 
   resultOOP = _blox_vm_proxy->nvmsgSend(stc->receiver, stc->selector, args,
 					1 + n_param_values);
