@@ -28,19 +28,8 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <setjmp.h>
-#include <sys/types.h>
-#include <errno.h>
-#include <string.h>
-#include <limits.h>
-
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-
 struct heap;
 
 /* Internal structure that defines the format of the heap descriptor.
