@@ -255,14 +255,6 @@ extern OOP _gst_find_an_instance (OOP class_oop) ATTRIBUTE_PURE;
 extern void _gst_swap_objects (OOP oop1,
 			       OOP oop2);
 
-/* Called after the fundamental class hierarchy has been defined, this
-   function goes through and fixes up all the objects in the oop table
-   that don't have a objClass (objClass == _gst_nil_oop).  It's a
-   chicken-and-egg problem: the _gst_metaclass_class doesn't yet exist
-   when the hierarchy is put together, so after it's created, we have
-   to go back and fix all the metaclasses that we created. */
-extern void _gst_fixup_metaclass_objects (void);
-
 /* Flip the two survivor spaces.  Starting from the root set, move eden
    objects to survivor space, tenuring objects when the top of the space
    is hit.  Then tell the incremental sweeper not to sweep old objects.  */
