@@ -34,7 +34,13 @@
 #define __lightning_core_common_h_
 
 typedef struct {
-  jit_insn		 *pc;
+  union {
+    jit_insn             *pc;
+    _uc                  *uc_pc;
+    _us                  *us_pc;
+    _ui                  *ui_pc;
+    _ul                  *ul_pc;
+  }                       x;
   struct jit_fp		 *fp;
   struct jit_local_state jitl;
 } jit_state;
@@ -45,10 +51,10 @@ static jit_state 			_jit;
 
 #define _jitl				_jit.jitl
 
-#define	jit_get_ip()			(*(jit_code *) &_jit.pc)
-#define	jit_set_ip(ptr)			(_jit.pc = (jit_insn *) ptr, jit_get_ip())
-#define	jit_get_label()			(_jit.pc)
-#define	jit_forward()			(_jit.pc)
+#define	jit_get_ip()			(*(jit_code *) &_jit.x.pc)
+#define	jit_set_ip(ptr)			(_jit.x.pc = (jit_insn *) ptr, jit_get_ip())
+#define	jit_get_label()			(_jit.x.pc)
+#define	jit_forward()			(_jit.x.pc)
 
 #define	jit_field(struc, f)		( ((long) (&((struc *) 8)->f) ) - 8)
 #define	jit_ptr_field(struc_p, f)	( ((long) (&((struc_p) 8)->f) ) - 8)
