@@ -61,6 +61,8 @@ static int do_printfv (STREAM *stream, const char *format, union printf_arg cons
 #define ASCII_DEL	(int)'\177'
 #define ASCII_SPACE	(int)' '
 
+#define IS_MODIFIER(spec)  (!((spec)->fmt))
+
 /* TODO:  This is not thread-safe.  Change the API to pass the spec_table
           in as the first parameter to the functions which use it? */
 static spec_entry *spec_table[ASCII_DEL - ASCII_SPACE];
@@ -135,12 +137,14 @@ register_printf_function (unsigned spec, printf_function *fmt, printf_arginfo_fu
 {
   spec_entry *new, *old;
   old = spec_lookup (spec);
-  if (old && old->modifier_char)
+  if (old && IS_MODIFIER (old))
+    return NULL;
+
+  if (!fmt || !spec)
     return NULL;
 
   new = snv_new (spec_entry, 1);
   new->spec = spec;
-  new->modifier_char = FALSE;
   new->fmt = fmt;
   new->arg = arg;
   new->user = NULL;
@@ -154,10 +158,11 @@ static int
 call_argtype_function (struct printf_info *const pinfo, int **argtypes, spec_entry *spec)
 {
   int n;
-  int argindex = (pinfo->dollar && !spec->modifier_char)
+  int argindex = (pinfo->dollar && !IS_MODIFIER (spec))
          ? pinfo->dollar - 1
          : pinfo->argindex;
 
+  int save_argindex = pinfo->argindex;
   int save_state = pinfo->state;
   char const *save_format = pinfo->format;
 
@@ -208,6 +213,7 @@ call_argtype_function (struct printf_info *const pinfo, int **argtypes, spec_ent
 	  pinfo->argc = argindex + n;
 
 	  /* Call again... */
+	  pinfo->argindex = save_argindex;
 	  pinfo->format = save_format;
 	  pinfo->state = save_state;
           pinfo->spec = *pinfo->format;
@@ -217,7 +223,7 @@ call_argtype_function (struct printf_info *const pinfo, int **argtypes, spec_ent
 	}
     }
 
-  if (!pinfo->dollar || spec->modifier_char)
+  if (!pinfo->dollar && !IS_MODIFIER (spec))
     pinfo->argindex += n;
 
   return n;
@@ -243,10 +249,11 @@ printf_strerror (void)
 
 /* (re)initialise the memory used by PPARSER. */
 static inline void
-parser_init (struct printf_info *pinfo, const char *format)
+parser_init (struct printf_info *pinfo, const char *format, const union printf_arg *args)
 {
   memset (pinfo, 0, sizeof (struct printf_info));
   pinfo->format = format;
+  pinfo->args = args;
 }
 
 static inline struct printf_info *
@@ -340,7 +347,7 @@ parse_printf_format (const char *format, int n, int *argtypes)
 
   return_val_if_fail (format != NULL, -1);
 
-  parser_init (&info, format);
+  parser_init (&info, format, NULL);
 
   while (*info.format != EOS)
     {
@@ -372,14 +379,14 @@ parse_printf_format (const char *format, int n, int *argtypes)
 		      goto error;
 		    }
 
-		  if (!spec->modifier_char &&
+		  if (!IS_MODIFIER (spec) &&
 		      !(info.state & (SNV_STATE_BEGIN | SNV_STATE_SPECIFIER)))
 		    {
 		      PRINTF_ERROR (&info, "invalid combination of flags");
 		      goto error;
 		    }
 
-	          argindex = info.dollar && !spec->modifier_char
+	          argindex = info.dollar && !IS_MODIFIER (spec)
 			       ? info.dollar - 1 : info.argindex;
 
 		  /* ...and call the relevant handler.  */
@@ -401,12 +408,12 @@ parse_printf_format (const char *format, int n, int *argtypes)
 		    goto error;
 
 		  info.argc = MAX (info.argc, argindex + status);
-		  if (!info.dollar || spec->modifier_char)
+		  if (!info.dollar && !IS_MODIFIER (spec))
 		    info.argindex += status;
 
 		  info.format++;
 		}
-	      while (spec->modifier_char);
+	      while (IS_MODIFIER (spec));
 	      continue;
 	    }
 
@@ -446,7 +453,7 @@ do_printfv (STREAM *stream, const char *format, union printf_arg const args[])
      Here we scan through the format string and move bytes into the
      stream and call handlers based on the parser state. */
 
-  parser_init (&info, format);
+  parser_init (&info, format, args);
 
   /* Keep going until the format string runs out! */
   while (*info.format != EOS)
@@ -478,7 +485,7 @@ do_printfv (STREAM *stream, const char *format, union printf_arg const args[])
 		      goto error;
 		    }
 
-		  if (!spec->modifier_char &&
+		  if (!IS_MODIFIER (spec) &&
 		      !(info.state & (SNV_STATE_BEGIN | SNV_STATE_SPECIFIER)))
 		    {
 		      PRINTF_ERROR (&info, "invalid combination of flags");
@@ -495,22 +502,22 @@ do_printfv (STREAM *stream, const char *format, union printf_arg const args[])
 		  if (status < 0)
 		    goto error;
 
-		  argindex = info.dollar && !spec->modifier_char
+		  argindex = info.dollar && !IS_MODIFIER (spec)
 		    ? info.dollar - 1 : info.argindex;
 
 		  info.format++;
 		  info.argc = MAX (info.argc, argindex + status);
-		  if (!info.dollar || spec->modifier_char)
+		  if (!info.dollar && !IS_MODIFIER (spec))
 		    info.argindex += status;
-
-		  status = (*spec->fmt) (stream, &info, args + argindex);
-
-		  if (status < 0)
-		    goto error;
-
-		  info.count += status;
 		}
-	      while (info.count >= 0 && spec->modifier_char);
+	      while (info.count >= 0 && IS_MODIFIER (spec));
+
+	      status = (*spec->fmt) (stream, &info, args + argindex);
+
+	      if (status < 0)
+	        goto error;
+
+	      info.count += status;
 	      continue;
 	    }
 
@@ -566,7 +573,7 @@ stream_printfv (STREAM *stream, const char *format, snv_constpointer const *ap)
 
   return_val_if_fail (format != NULL, SNV_ERROR);
 
-  parser_init (&info, format);
+  parser_init (&info, format, NULL);
   while (*info.format != EOS)
     {
       int ch = (int) *info.format++;
@@ -594,7 +601,7 @@ stream_printfv (STREAM *stream, const char *format, snv_constpointer const *ap)
                      goto error;
                    }
 
-                 if (!spec->modifier_char &&
+                 if (!IS_MODIFIER (spec) &&
                      !(info.state & (SNV_STATE_BEGIN | SNV_STATE_SPECIFIER)))
                    {
                      PRINTF_ERROR (&info, "invalid combination of flags");
@@ -607,7 +614,7 @@ stream_printfv (STREAM *stream, const char *format, snv_constpointer const *ap)
 
                  info.format++;
                }
-             while (info.count >= 0 && spec->modifier_char);
+             while (info.count >= 0 && IS_MODIFIER (spec));
              continue;
            }
          /* An escaped CHAR_SPEC: ignore it (by falling through). */
@@ -666,7 +673,7 @@ stream_printfv (STREAM *stream, const char *format, snv_constpointer const *ap)
 
           case PA_DOUBLE|PA_FLAG_LONG_DOUBLE:
 #ifdef HAVE_LONG_DOUBLE
-           args[index].pa_long_double = **(const long double **)(ap + index);
+           args[index].pa_long_double = **(const snv_long_double **)(ap + index);
             break;
 #endif
            /* else fall through */
@@ -739,7 +746,7 @@ stream_vprintf (STREAM *stream, const char *format, va_list ap)
 
   return_val_if_fail (format != NULL, SNV_ERROR);
 
-  parser_init (&info, format);
+  parser_init (&info, format, NULL);
   while (*info.format != EOS)
     {
       int ch = (int) *info.format++;
@@ -767,7 +774,7 @@ stream_vprintf (STREAM *stream, const char *format, va_list ap)
 		      goto error;
 		    }
 
-		  if (!spec->modifier_char &&
+		  if (!IS_MODIFIER (spec) &&
 		      !(info.state & (SNV_STATE_BEGIN | SNV_STATE_SPECIFIER)))
 		    {
 		      PRINTF_ERROR (&info, "invalid combination of flags");
@@ -780,7 +787,7 @@ stream_vprintf (STREAM *stream, const char *format, va_list ap)
 
 		  info.format++;
 		}
-	      while (info.count >= 0 && spec->modifier_char);
+	      while (info.count >= 0 && IS_MODIFIER (spec));
 	      continue;
 	    }
 	  /* An escaped CHAR_SPEC: ignore it (by falling through). */
@@ -1035,6 +1042,13 @@ snv_fileputc (int ch, STREAM *stream)
   return putc (ch, fp);
 }
 
+static int
+snv_fileputc_unlocked (int ch, STREAM *stream)
+{
+  FILE *fp = (FILE *) stream_details (stream);
+  return SNV_PUTC_UNLOCKED (ch, fp);
+}
+
 /**
  * snv_printf: printf.h
  * @format: a % delimited format string.
@@ -1076,9 +1090,12 @@ int
 snv_vprintf (const char *format, va_list ap)
 {
   int result;
-  STREAM *out = stream_new (stdout, SNV_UNLIMITED, NULL, snv_fileputc);
+  STREAM *out = stream_new (stdout, SNV_UNLIMITED, NULL, snv_fileputc_unlocked);
+  int tmp;
+
+  SNV_WITH_LOCKED_FP (stdout, tmp)
+    result = stream_vprintf (out, format, ap);
 
-  result = stream_vprintf (out, format, ap);
   stream_delete (out);
   return result;
 }
@@ -1099,9 +1116,11 @@ int
 snv_printfv (const char *format, snv_constpointer const args[])
 {
   int result;
-  STREAM *out = stream_new (stdout, SNV_UNLIMITED, NULL, snv_fileputc);
+  STREAM *out = stream_new (stdout, SNV_UNLIMITED, NULL, snv_fileputc_unlocked);
+  int tmp;
 
-  result = stream_printfv (out, format, args);
+  SNV_WITH_LOCKED_FP (stdout, tmp)
+    result = stream_printfv (out, format, args);
   stream_delete (out);
   return result;
 }
@@ -1149,9 +1168,11 @@ int
 snv_vfprintf (FILE *file, const char *format, va_list ap)
 {
   int result;
-  STREAM *out = stream_new (file, SNV_UNLIMITED, NULL, snv_fileputc);
+  STREAM *out = stream_new (file, SNV_UNLIMITED, NULL, snv_fileputc_unlocked);
+  int tmp;
 
-  result = stream_vprintf (out, format, ap);
+  SNV_WITH_LOCKED_FP (file, tmp)
+    result = stream_vprintf (out, format, ap);
   stream_delete (out);
   return result;
 }
@@ -1173,9 +1194,12 @@ int
 snv_fprintfv (FILE *file, const char *format, snv_constpointer const args[])
 {
   int result;
-  STREAM *out = stream_new (file, SNV_UNLIMITED, NULL, snv_fileputc);
+  STREAM *out = stream_new (file, SNV_UNLIMITED, NULL, snv_fileputc_unlocked);
+  int tmp;
+
+  SNV_WITH_LOCKED_FP (file, tmp)
+    result = stream_printfv (out, format, args);
 
-  result = stream_printfv (out, format, args);
   stream_delete (out);
   return result;
 }
